@@ -4,13 +4,14 @@ from distutils.core import Extension
 try:
     from Cython.Distutils import build_ext as build_pyx
     import Cython.Compiler.Version
-    print("Building with Cython %s." % Cython.Compiler.Version.version)
     CYTHON_INSTALLED = True
 except ImportError:
     CYTHON_INSTALLED = False
 
 EXT_MODULES = ["lxml.etree", "lxml.objectify","lxml.pyclasslookup"]
 
+PACKAGE_PATH = "src/lxml/"
+
 def env_var(name):
     value = os.getenv(name, '')
     return value.split(os.pathsep)
@@ -18,23 +19,42 @@ def env_var(name):
 def ext_modules(static_include_dirs, static_library_dirs, static_cflags): 
     if CYTHON_INSTALLED:
         source_extension = ".pyx"
+        print("Building with Cython %s." % Cython.Compiler.Version.version)
     else:
         print ("NOTE: Trying to build without Cython, pre-generated "
-               "'src/lxml/etree.c' needs to be available.")
+               "'%slxml.etree.c' needs to be available." % PACKAGE_PATH)
         source_extension = ".c"
 
     if OPTION_WITHOUT_OBJECTIFY:
         modules = [ entry for entry in EXT_MODULES
-                    if 'objectify' not in entry[0] ]
+                    if 'objectify' not in entry ]
     else:
         modules = EXT_MODULES
 
+    lib_versions = get_library_versions()
+    if lib_versions[0]:
+        print("Using build configuration of libxml2 %s and libxslt %s" % 
+              lib_versions)
+    else:
+        print("Using build configuration of libxslt %s" % 
+              lib_versions[1])
+
     _include_dirs = include_dirs(static_include_dirs)
     _library_dirs = library_dirs(static_library_dirs)
     _cflags = cflags(static_cflags)
     _define_macros = define_macros()
     _libraries = libraries()
 
+    if _library_dirs:
+        message = "Building against libxml2/libxslt in "
+        if len(_library_dirs) > 1:
+            print(message + "one of the following directories:")
+            for dir in _library_dirs:
+                print("  " + dir)
+        else:
+            print(message + "the following directory: " +
+                  _library_dirs[0])
+
     if OPTION_AUTO_RPATH:
         runtime_library_dirs = _library_dirs
     else:
@@ -42,10 +62,11 @@ def ext_modules(static_include_dirs, static_library_dirs, static_cflags):
     
     result = []
     for module in modules:
+        main_module_source = PACKAGE_PATH + module + source_extension
         result.append(
             Extension(
             module,
-            sources = ["src/lxml/" + module + source_extension],
+            sources = [main_module_source],
             extra_compile_args = ['-w'] + _cflags,
             define_macros = _define_macros,
             include_dirs = _include_dirs,
@@ -91,7 +112,6 @@ def include_dirs(static_include_dirs):
     if OPTION_STATIC:
         if not static_include_dirs:
             static_include_dirs = env_var('INCLUDE')
-        assert static_include_dirs, "Static build not configured, see doc/build.txt"
         return static_include_dirs
     # filter them from xslt-config --cflags
     result = []
@@ -109,15 +129,21 @@ def cflags(static_cflags):
     if OPTION_STATIC:
         if not static_cflags:
             static_cflags = env_var('CFLAGS')
-        assert static_cflags, "Static build not configured, see doc/build.txt"
         result.extend(static_cflags)
-        return result
+    else:
+        # anything from xslt-config --cflags that doesn't start with -I
+        possible_cflags = flags('cflags')
+        for possible_cflag in possible_cflags:
+            if not possible_cflag.startswith('-I'):
+                result.append(possible_cflag)
+
+    if sys.platform in ('darwin',):
+        for opt in result:
+            if 'flat_namespace' in opt:
+                break
+        else:
+            result.append('-flat_namespace')
 
-    # anything from xslt-config --cflags that doesn't start with -I
-    possible_cflags = flags('cflags')
-    for possible_cflag in possible_cflags:
-        if not possible_cflag.startswith('-I'):
-            result.append(possible_cflag)
     return result
 
 def define_macros():
@@ -128,8 +154,13 @@ def define_macros():
         macros.append(('WITHOUT_THREADING', None))
     return macros
 
-def flags(option):
-    cmd = "%s --%s" % (find_xslt_config(), option)
+_ERROR_PRINTED = False
+
+def run_command(cmd, *args):
+    if not cmd:
+        return ''
+    if args:
+        cmd = ' '.join((cmd,) + args)
     try:
         import subprocess
     except ImportError:
@@ -141,12 +172,46 @@ def flags(option):
                              stdout=subprocess.PIPE, stderr=subprocess.PIPE)
         rf, ef = p.stdout, p.stderr
     errors = ef.read()
-    if errors:
+    global _ERROR_PRINTED
+    if errors and not _ERROR_PRINTED:
+        _ERROR_PRINTED = True
         print("ERROR: %s" % errors)
         print("** make sure the development packages of libxml2 and libxslt are installed **\n")
-    return str(rf.read()).split()
+    output = rf.read()
+    return (output or '').strip()
+
+def get_library_versions():
+    xml2_version = run_command(find_xml2_config(), "--version")
+    xslt_version = run_command(find_xslt_config(), "--version")
+    return xml2_version, xslt_version
+
+def flags(option):
+    xml2_flags = run_command(find_xml2_config(), "--%s" % option)
+    xslt_flags = run_command(find_xslt_config(), "--%s" % option)
+
+    flag_list = xml2_flags.split()
+    for flag in xslt_flags.split():
+        if flag not in flag_list:
+            flag_list.append(flag)
+    return flag_list
 
 XSLT_CONFIG = None
+XML2_CONFIG = None
+
+def find_xml2_config():
+    global XML2_CONFIG
+    if XML2_CONFIG:
+        return XML2_CONFIG
+    option = '--with-xml2-config='
+    for arg in sys.argv:
+        if arg.startswith(option):
+            sys.argv.remove(arg)
+            XML2_CONFIG = arg[len(option):]
+            return XML2_CONFIG
+    else:
+        # default: do nothing, rely only on xslt-config
+        XML2_CONFIG = os.getenv('XML2_CONFIG', '')
+    return XML2_CONFIG
 
 def find_xslt_config():
     global XSLT_CONFIG
@@ -159,7 +224,7 @@ def find_xslt_config():
             XSLT_CONFIG = arg[len(option):]
             return XSLT_CONFIG
     else:
-        XSLT_CONFIG = 'xslt-config'
+        XSLT_CONFIG = os.getenv('XSLT_CONFIG', 'xslt-config')
     return XSLT_CONFIG
 
 def has_option(name):
