@@ -1,10 +1,7 @@
 # -*- coding: utf-8 -*-
 
 """
-Tests specific to the extended etree API
-
-Tests that apply to the general ElementTree API should go into
-test_elementtree
+Tests for different Element class lookup mechanisms.
 """
 
 
@@ -23,25 +20,24 @@ xml_str = '''\
 </root>'''
 
 class ClassLookupTestCase(HelperTestCase):
-    """Test cases for lxml.elementlib.classlookup
+    """Test cases for different Element class lookup mechanisms.
     """
     etree = etree
 
     def tearDown(self):
         etree.setElementClassLookup()
-        etree.Namespace("myNS").clear()
-        etree.Namespace("otherNS").clear()
+        super(ClassLookupTestCase, self).tearDown()
 
     def test_namespace_lookup(self):
         class TestElement(etree.ElementBase):
             FIND_ME = "namespace class"
 
-        ns = etree.Namespace("myNS")
-        ns[None] = TestElement
-
         lookup = etree.ElementNamespaceClassLookup()
         etree.setElementClassLookup(lookup)
 
+        ns = lookup.get_namespace("myNS")
+        ns[None] = TestElement
+
         root = etree.XML(xml_str)
         self.assertEquals(root.FIND_ME,
                           TestElement.FIND_ME)
@@ -74,20 +70,6 @@ class ClassLookupTestCase(HelperTestCase):
         self.assertEquals("default pi", root[0].FIND_ME)
         self.assertEquals("default comment", root[1].FIND_ME)
 
-    def test_default_class_lookup_is_not_nslookup(self):
-        class TestElement(etree.ElementBase):
-            FIND_ME = "namespace class"
-
-        ns = etree.Namespace("myNS")
-        ns[None] = TestElement
-
-        lookup = etree.ElementDefaultClassLookup()
-        etree.setElementClassLookup(lookup)
-
-        root = etree.XML(xml_str)
-        self.assertFalse(hasattr(root, 'FIND_ME'))
-        self.assertFalse(hasattr(root[0][-1], 'FIND_ME'))
-
     def test_attribute_based_lookup(self):
         class TestElement(etree.ElementBase):
             FIND_ME = "attribute_based"
@@ -133,12 +115,12 @@ class ClassLookupTestCase(HelperTestCase):
                 if name == 'c1':
                     return TestElement1
 
-        ns = etree.Namespace("otherNS")
-        ns[None] = TestElement2
-
         lookup = etree.ElementNamespaceClassLookup( MyLookup() )
         etree.setElementClassLookup(lookup)
 
+        ns = lookup.get_namespace("otherNS")
+        ns[None] = TestElement2
+
         root = etree.XML(xml_str)
         self.assertFalse(hasattr(root, 'FIND_ME'))
         self.assertEquals(root[0].FIND_ME,
@@ -171,6 +153,26 @@ class ClassLookupTestCase(HelperTestCase):
         self.assertFalse(hasattr(root, 'FIND_ME'))
         self.assertFalse(hasattr(root[0], 'FIND_ME'))
 
+    def test_lookup_without_fallback(self):
+        class Lookup(etree.CustomElementClassLookup):
+             def __init__(self):
+                 # no super call here, so no fallback is set
+                 pass
+
+             def lookup(self, node_type, document, namespace, name):
+                 return Foo
+
+        class Foo(etree.ElementBase):
+             def custom(self):
+                 return "test"
+
+        parser = self.etree.XMLParser()
+        parser.set_element_class_lookup( Lookup() )
+
+        root = etree.XML('<foo/>', parser)
+
+        self.assertEquals("test", root.custom())
+
 
 def test_suite():
     suite = unittest.TestSuite()
@@ -178,4 +180,4 @@ def test_suite():
     return suite
 
 if __name__ == '__main__':
-    unittest.main()
+    print 'to test use test.py %s' % __file__
